package com.hammingweight.hammock;

import junit.framework.TestCase;

import org.easymock.EasyMock;

public class HammockTest extends TestCase {

	// The OUT.
	private Hammock hammock;

	public void setUp() {
		this.hammock = new Hammock();
	}

	// If a method is invoked without an expectation having been set we should
	// throw a HammockException indicating that the method was unexpected.
	public void testInvokeNoExpectationsSet() {
		MockMethod mm = new MockMethod("foo", 0, null, true);
		IMockObject mo = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mo, new Object[] {});

		// Exercise.
		try {
			this.hammock.invoke(mi);
			fail("Method invoked without expectation.");
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.METHOD_CANNOT_BE_EVALUATED, he
					.getError());
		}
	}

	// If a method couldn't be evaluated, we should throw an exception if verify
	// is called.
	public void testInvokeNoExpectationsSetVerify() {
		MockMethod mm = new MockMethod("foo", 0, null, true);
		IMockObject mo = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mo, new Object[] {});
		try {
			this.hammock.invoke(mi);
		} catch (HammockException he) {
		}

		// Exercise.
		try {
			this.hammock.verify();
			fail("Method invoked without expectation followed by verify.");
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.METHOD_CANNOT_BE_EVALUATED, he
					.getError());
		}
	}

	// Test that verify without any expectations and without any invocations
	// succeeds.
	public void testVerifyNoExpectationsNoInvocations() {
		this.hammock.verify();
	}

	// Test that we get a method handler if we set an expectation that a method
	// will be called.
	public void testSetExpectation1() {
		MockMethod mm = new MockMethod("foo", 0, null, true);
		MethodHandler mh = this.hammock.setExpectation(mm);
		assertNotNull(mh);
	}

	// Test that we get a method handler if we set an expectation that a method
	// will be called with expected arguments.
	public void testSetExpectation2() {
		MockMethod mm = new MockMethod("foo", 1, null, true);
		MethodHandler mh = this.hammock.setExpectation(mm,
				new Object[] { "hello" });
		assertNotNull(mh);
	}

	// Test that we get a method handler if we set an expectation that a method
	// will be called with expected arguments and an expected target.
	public void testSetExpectation3() {
		MockMethod mm = new MockMethod("foo", 1, null, true);
		IMockObject mo = EasyMock.createMock(IMockObject.class);
		MethodHandler mh = this.hammock.setExpectation(mm, mo,
				new Object[] { "hello" });
		assertNotNull(mh);
	}

	// Test that if we set an expectation that invocation works.
	public void testSetExpectationInvocation() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		this.hammock.setExpectation(mm).setReturnValue("bar");
		IMockObject mo = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mo,
				new Object[] { "foo" });
		this.hammock.invoke(mi);
		assertEquals("bar", mi.getReturnValue());
	}

	// Test that if we set two expectations that we can verify execute the
	// second invocation.
	public void testSetTwoExpectationsInvocation() throws Throwable {
		MockMethod mm = new MockMethod("foo", 0, null, true);
		this.hammock.setExpectation(mm);
		MockMethod mm2 = new MockMethod("bar", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		this.hammock.setExpectation(mm2).setReturnValue("baz");
		IMockObject mo = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm2, mo,
				new Object[] { "foo" });
		this.hammock.invoke(mi);
		assertEquals("baz", mi.getReturnValue());
	}

	// We check that if we set an expectation, we invoke the expected method and
	// we verify that everything's OK
	public void testSetExpectationInvokeVerify() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		this.hammock.setExpectation(mm).setReturnValue("bar");
		IMockObject mo = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mo,
				new Object[] { "foo" });
		this.hammock.invoke(mi);
		this.hammock.verify();
	}

	// We check that if we set an expectation, we don't invoke the expected
	// method and we verify that we get an exception.
	public void testSetExpectationDontInvokeVerify() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		this.hammock.setExpectation(mm).setReturnValue("bar");
		try {
			this.hammock.verify();
			fail("A method was never invoked but verification was OK.");
		} catch (HammockException he) {
			assertEquals(
					IHammockExceptionErrors.METHOD_INVOKED_UNEXPECTED_NUMBER_OF_TIMES,
					he.getError());
		}
	}

	// We check that if we set two expectation, we invoke the expected methods
	// and
	// we verify that everything's OK
	public void testSetTwoExpectationInvokeVerify() throws Throwable {
		MockMethod mm = new MockMethod("foo", 0, null, true);
		this.hammock.setExpectation(mm);
		MockMethod mm2 = new MockMethod("bar", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		this.hammock.setExpectation(mm2).setReturnValue("baz");
		IMockObject mo = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm2, mo,
				new Object[] { "foo" });
		this.hammock.invoke(mi);
		MethodInvocation mi2 = new MethodInvocation(mm, mo, new Object[] {});
		this.hammock.invoke(mi2);
		assertEquals("baz", mi.getReturnValue());
		this.hammock.verify();
	}

	// Test that if we use a strict mock that the methods must invoked in the
	// expected sequence.
	public void testStrictMock() throws Throwable {
		this.hammock.setStrictOrdering();
		MockMethod mm = new MockMethod("foo", 0, null, true);
		this.hammock.setExpectation(mm);
		MockMethod mm2 = new MockMethod("bar", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		this.hammock.setExpectation(mm2).setReturnValue("baz");
		IMockObject mo = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm2, mo,
				new Object[] { "foo" });
		try {
			this.hammock.invoke(mi);
			fail("Strict ordering wasn't enforced.");
		} catch (HammockException he) {
			assertEquals(
					IHammockExceptionErrors.METHOD_INVOKED_OUT_OF_SEQUENCE, he
							.getError());
		}
	}

	// Test that if we use a strict mock and that we don't invoke the methods in
	// sequence that verification will fail.
	public void testStrictMockVerify() throws Throwable {
		this.hammock.setStrictOrdering();
		MockMethod mm = new MockMethod("foo", 0, null, true);
		this.hammock.setExpectation(mm);
		MockMethod mm2 = new MockMethod("bar", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		this.hammock.setExpectation(mm2).setReturnValue("baz");
		IMockObject mo = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm2, mo,
				new Object[] { "foo" });
		try {
			this.hammock.invoke(mi);
			fail("Mock isn't strict.");
		} catch (HammockException he) {
			// This should happen.
		}

		// Now invoke the methods in the expected sequence.
		MethodInvocation mi2 = new MethodInvocation(mm, mo, new Object[] {});
		this.hammock.invoke(mi2);
		this.hammock.invoke(mi);

		try {
			this.hammock.verify();
			fail("Verification should fail if a method was invoked out of sequence.");
		} catch (HammockException he) {
			assertEquals(
					IHammockExceptionErrors.METHOD_INVOKED_OUT_OF_SEQUENCE, he
							.getError());
		}
	}

	// Test that argument matching is OK.
	public void testArgumentMatching() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		this.hammock.setExpectation(mm, new Object[] { "hello" })
				.setReturnValue("goodbye");

		IMockObject mo = EasyMock.createMock(IMockObject.class);
		try {
			this.hammock.invoke(new MethodInvocation(mm, mo,
					new Object[] { "HELLO" }));
			fail("We managed to invoke a method with the incorrect arguments.");
		} catch (HammockException he) {

		}

		MethodInvocation mi = new MethodInvocation(mm, mo,
				new Object[] { "hello" });
		this.hammock.invoke(mi);
		assertEquals("goodbye", mi.getReturnValue());
	}

	// Test that the mock object is matched.
	public void testMockObjectMatching() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		IMockObject mo = EasyMock.createMock(IMockObject.class);
		this.hammock.setExpectation(mm, mo, new Object[] { "hello" })
				.setReturnValue("goodbye");
		IMockObject mo2 = EasyMock.createMock(IMockObject.class);
		try {
			this.hammock.invoke(new MethodInvocation(mm, mo2,
					new Object[] { "hello" }));
			fail("We managed to invoke a method on the incorrect mock.");
		} catch (HammockException he) {

		}

		MethodInvocation mi = new MethodInvocation(mm, mo,
				new Object[] { "hello" });
		this.hammock.invoke(mi);
		assertEquals("goodbye", mi.getReturnValue());
	}

	// Test that a stub expectation allows a method to be invoked an arbitrary
	// number of times.
	public void testSetStubExpectation() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.INTEGER_CLASS, true);
		IMockObject mo = EasyMock.createMock(IMockObject.class);

		this.hammock.setStubExpectation(mm).setReturnValue(new Integer(9));

		// A stubbed expectation can be satisfied even if the method isn't
		// invoked at all.
		this.hammock.verify();

		for (int i = 0; i < 100; i++) {
			MethodInvocation mi = new MethodInvocation(mm, mo,
					new Object[] { "hello" });
			this.hammock.invoke(mi);
			assertEquals(new Integer(9), mi.getReturnValue());
		}
	}

	// Specify arguments for a stubbed expectation.
	public void testSetStubExpectationWithArgument() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.INTEGER_CLASS, true);
		IMockObject mo = EasyMock.createMock(IMockObject.class);

		this.hammock.setStubExpectation(mm, new Object[] { "hello" })
				.setReturnValue(new Integer(9));
		this.hammock.verify();
		for (int i = 0; i < 100; i++) {
			MethodInvocation mi = new MethodInvocation(mm, mo,
					new Object[] { "hello" });
			this.hammock.invoke(mi);
			assertEquals(new Integer(9), mi.getReturnValue());
			mi = new MethodInvocation(mm, mo, new Object[] { "good bye" });
			try {
				this.hammock.invoke(mi);
				fail("Shouldn't accept argument");
			} catch (HammockException he) {

			}
		}
	}

	// Test a stubbed expectation with arguments and a mock object,
	public void testSetStubExpectationWithObject() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.INTEGER_CLASS, true);
		IMockObject mo1 = EasyMock.createMock(IMockObject.class);
		IMockObject mo2 = EasyMock.createMock(IMockObject.class);

		this.hammock.setStubExpectation(mm, mo1, new Object[] { "hello" })
				.setReturnValue(new Integer(9));
		this.hammock.verify();
		for (int i = 0; i < 100; i++) {
			MethodInvocation mi = new MethodInvocation(mm, mo1,
					new Object[] { "hello" });
			this.hammock.invoke(mi);
			assertEquals(new Integer(9), mi.getReturnValue());

			// Try with the wrong object.
			mi = new MethodInvocation(mm, mo2, new Object[] { "hello" });
			try {
				this.hammock.invoke(mi);
				fail("Shouldn't accept object");
			} catch (HammockException he) {

			}
		}
	}

	// Test an expectation with method and object specified but no expected
	// arguments.
	public void testExpectationForMethodObject() {
		MockMethod mm = new MockMethod("foo", 1, null, true);
		IMockObject mo1 = EasyMock.createMock(IMockObject.class);
		IMockObject mo2 = EasyMock.createMock(IMockObject.class);
		this.hammock.setExpectation(mm, mo1).setInvocationCount(1, 2);
		MethodInvocation mi1 = new MethodInvocation(mm, mo1,
				new Object[] { "foo" });
		this.hammock.invoke(mi1);
		assertTrue(mi1.isEvaluated());
		MethodInvocation mi2 = new MethodInvocation(mm, mo2,
				new Object[] { "foo" });
		try {
			this.hammock.invoke(mi2);
			fail("No expectation set for mock object mo2");
		} catch (HammockException e) {
		}
		assertFalse(mi2.isEvaluated());
	}

	// Test with a stubbed expectation with method and object specified but no
	// expected arguments.
	public void testStubExpectationForMethodObject() {
		MockMethod mm = new MockMethod("foo", 1, null, true);
		IMockObject mo1 = EasyMock.createMock(IMockObject.class);
		IMockObject mo2 = EasyMock.createMock(IMockObject.class);
		this.hammock.setStubExpectation(mm, mo1);
		MethodInvocation mi2 = new MethodInvocation(mm, mo2,
				new Object[] { "foo" });
		assertFalse(mi2.isEvaluated());
		try {
			this.hammock.invoke(mi2);
			fail("No expectation set for mock object mo2");
		} catch (HammockException e) {
		}
		MethodInvocation mi1 = new MethodInvocation(mm, mo1,
				new Object[] { "foo" });
		this.hammock.invoke(mi1);
		assertTrue(mi1.isEvaluated());
	}

	// Test the isVerified method - false
	public void testIsVerifiedFalse() {
		MockMethod mm = new MockMethod("foo", 1, null, true);
		this.hammock.setExpectation(mm);
		assertFalse(this.hammock.isVerified());
	}

	// Test the isVerified method - true
	public void testIsVerifiedTrue() {
		MockMethod mm = new MockMethod("foo", 1, null, true);
		this.hammock.setExpectation(mm);
		IMockObject mo = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mo,
				new Object[] { "foo" });
		this.hammock.invoke(mi);
		assertTrue(this.hammock.isVerified());
	}
}
