package com.hammingweight.hammock;

import junit.framework.TestCase;

import org.easymock.EasyMock;

public class HamspyTest extends TestCase {

	// The OUT.
	private Hamspy hamspy = new Hamspy();

	public void testInvokeVoidAbstractMethod() throws Throwable {
		// Method that takes 1 argument, is void and abstract
		MockMethod mm = new MockMethod("foo", 1, null, true);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });

		this.hamspy.invoke(mi);

		// The method should have been evaluated.
		assertTrue(mi.isEvaluated());

		// For a void return type, the return value is null.
		assertNull(mi.getReturnValue());
	}

	public void testInvokeVoidNonAbstractMethod() throws Throwable {
		// Method that takes 1 argument, is void and non-abstract
		MockMethod mm = new MockMethod("foo", 1, null, false);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });

		this.hamspy.invoke(mi);

		// The method should not have been evaluated since it is not abstract.
		assertFalse(mi.isEvaluated());
	}

	public void testDontSetReturnValueTwice() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.INTEGER_CLASS, true);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });

		// Set the return value. Hamspy shouldn't set it since the method has
		// been evaluated.
		mi.setReturnValue(new Integer(2));

		this.hamspy.invoke(mi);

		// Check that we get the return value that we set, not the default value
		// of 0.
		assertEquals(new Integer(2), mi.getReturnValue());
	}

	// The default boolean value returned by Hamspy should be false.
	public void testGetDefaultBooleanValue() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.BOOLEAN_CLASS, true);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);

		assertEquals(new Boolean(false), mi.getReturnValue());
	}

	// The default byte value is 0.
	public void testGetDefaultByteValue() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1, IClassDefinitions.BYTE_CLASS,
				true);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);

		assertEquals(new Byte((byte) 0), mi.getReturnValue());
	}

	// The default character value is 0.
	public void testGetDefaultCharacterValue() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.CHARACTER_CLASS, true);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);

		assertEquals(new Character((char) 0), mi.getReturnValue());
	}

	// The default int value is 0.
	public void testGetDefaultIntValue() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.INTEGER_CLASS, true);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);

		assertEquals(new Integer(0), mi.getReturnValue());
	}

	// The default long value is 0.
	public void testGetDefaultLongValue() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1, IClassDefinitions.LONG_CLASS,
				true);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);

		assertEquals(new Long(0), mi.getReturnValue());
	}

	// The default short value is 0.
	public void testGetDefaultShortValue() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1, IClassDefinitions.SHORT_CLASS,
				true);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);

		assertEquals(new Short((short) 0), mi.getReturnValue());
	}

	// The default Object value is null.
	public void testGetDefaultObjectValue() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);

		assertNull(mi.getReturnValue());
	}

	// Check that the getInvocations() method returns the correct number of
	// method invocations.
	public void testGetInvocationCount() {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		assertEquals(0, this.hamspy.getInvocations().length);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);
		assertEquals(1, this.hamspy.getInvocations().length);
	}

	// Check that the getInvocations() method returns the correct method
	// invocations.
	public void testGetInvocations() {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		assertEquals(0, this.hamspy.getInvocations().length);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);
		assertEquals(1, this.hamspy.getInvocations().length);
		assertEquals(mi, this.hamspy.getInvocations()[0]);
		this.hamspy.invoke(mi);
		assertEquals(2, this.hamspy.getInvocations().length);
		assertEquals(mi, this.hamspy.getInvocations()[1]);
	}

	public void testGetInvocationCountForMethod() {
		MockMethod mm1 = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		MockMethod mm2 = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		assertEquals(0, this.hamspy.getInvocations(mm1).length);
		assertEquals(0, this.hamspy.getInvocations(mm2).length);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi1 = new MethodInvocation(mm1, mockObject,
				new Object[] { null });
		MethodInvocation mi2 = new MethodInvocation(mm2, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi1);
		assertEquals(1, this.hamspy.getInvocations(mm1).length);
		assertEquals(0, this.hamspy.getInvocations(mm2).length);
		this.hamspy.invoke(mi2);
		assertEquals(1, this.hamspy.getInvocations(mm1).length);
		assertEquals(1, this.hamspy.getInvocations(mm2).length);
		this.hamspy.invoke(mi2);
		assertEquals(1, this.hamspy.getInvocations(mm1).length);
		assertEquals(2, this.hamspy.getInvocations(mm2).length);
		assertEquals(1, this.hamspy.getExpectation(mm1).getInvocationCount());
		assertEquals(2, this.hamspy.getExpectation(mm2).getInvocationCount());
	}

	public void testGetInvocationCountForExpectedMethod() {
		MockMethod mm1 = new MockMethod("foo", 1, null, true);
		MockMethod mm2 = new MockMethod("foo", 1, null, true);
		assertEquals(0, this.hamspy.getInvocations(mm1).length);
		assertEquals(0, this.hamspy.getInvocations(mm2).length);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		this.hamspy.setStubExpectation(mm1, mockObject, new Object[] { null });
		this.hamspy.setStubExpectation(mm2, mockObject, new Object[] { null });
		MethodInvocation mi1 = new MethodInvocation(mm1, mockObject,
				new Object[] { null });
		MethodInvocation mi2 = new MethodInvocation(mm2, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi1);
		assertEquals(1, this.hamspy.getInvocations(mm1).length);
		assertEquals(0, this.hamspy.getInvocations(mm2).length);
		this.hamspy.invoke(mi2);
		assertEquals(1, this.hamspy.getInvocations(mm1).length);
		assertEquals(1, this.hamspy.getInvocations(mm2).length);
		this.hamspy.invoke(mi2);
		assertEquals(2, this.hamspy.getInvocations(mm2).length);
	}

	public void testGetInvocationsForMethod() {
		MockMethod mm1 = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		MockMethod mm2 = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi1 = new MethodInvocation(mm1, mockObject,
				new Object[] { null });
		MethodInvocation mi2 = new MethodInvocation(mm2, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi1);
		assertEquals(mi1, this.hamspy.getInvocations(mm1)[0]);
		assertEquals(0, this.hamspy.getInvocations(mm2).length);
		this.hamspy.invoke(mi2);
		assertEquals(mi2, this.hamspy.getInvocations(mm2)[0]);
		this.hamspy.invoke(mi2);
		assertEquals(mi2, this.hamspy.getInvocations(mm2)[1]);
	}

	public void testGetInvocationsForMethodObject() {
		MockMethod mm1 = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		MockMethod mm2 = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		IMockObject mockObject1 = EasyMock.createMock(IMockObject.class);
		IMockObject mockObject2 = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi11a = new MethodInvocation(mm1, mockObject1,
				new Object[] { null });
		MethodInvocation mi11b = new MethodInvocation(mm1, mockObject1,
				new Object[] { null });
		MethodInvocation mi12 = new MethodInvocation(mm1, mockObject2,
				new Object[] { null });
		MethodInvocation mi21 = new MethodInvocation(mm2, mockObject1,
				new Object[] { null });
		this.hamspy.invoke(mi11a);
		this.hamspy.invoke(mi11b);
		this.hamspy.invoke(mi12);
		this.hamspy.invoke(mi21);
		assertEquals(2, this.hamspy.getInvocations(mm1, mockObject1).length);
		assertEquals(2, this.hamspy.getExpectation(mm1, mockObject1)
				.getInvocationCount());
		assertEquals(1, this.hamspy.getInvocations(mm1, mockObject2).length);
		assertEquals(1, this.hamspy.getExpectation(mm1, mockObject2)
				.getInvocationCount());
		assertEquals(1, this.hamspy.getInvocations(mm2, mockObject1).length);
		assertEquals(1, this.hamspy.getExpectation(mm2, mockObject1)
				.getInvocationCount());
		assertEquals(0, this.hamspy.getInvocations(mm2, mockObject2).length);
		assertEquals(0, this.hamspy.getExpectation(mm2, mockObject2)
				.getInvocationCount());

		assertEquals(mi11a, this.hamspy.getInvocations(mm1, mockObject1)[0]);
		assertEquals(mi11b, this.hamspy.getInvocations(mm1, mockObject1)[1]);
		assertEquals(mi21, this.hamspy.getInvocations(mm2, mockObject1)[0]);
		assertEquals(mi12, this.hamspy.getInvocations(mm1, mockObject2)[0]);
		assertEquals(0, this.hamspy.getInvocations(mm2, mockObject2).length);
	}

	public void testSetExpectation() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		MethodHandler mh = this.hamspy.setExpectation(mm);
		assertNotNull(mh);
		mh.setReturnValue("bar");
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		assertFalse(mi.isEvaluated());
		this.hamspy.invoke(mi);
		assertEquals("bar", mi.getReturnValue());
	}

	public void testSetTwoExpectations() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		this.hamspy.setExpectation(mm).setReturnValue("1");
		this.hamspy.setExpectation(mm).setReturnValue("2");
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);
		assertEquals("1", mi.getReturnValue());
		mi = new MethodInvocation(mm, mockObject, new Object[] { null });
		this.hamspy.invoke(mi);
		assertEquals("2", mi.getReturnValue());
	}

	public void testSetExpectationsWithArguments() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		this.hamspy.setExpectation(mm, new Object[] { "1" })
				.setReturnValue("1");
		this.hamspy.setExpectation(mm, new Object[] { "2" })
				.setReturnValue("2");
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { "2" });
		this.hamspy.invoke(mi);
		assertEquals("2", mi.getReturnValue());
		mi = new MethodInvocation(mm, mockObject, new Object[] { "1" });
		this.hamspy.invoke(mi);
		assertEquals("1", mi.getReturnValue());
		mi = new MethodInvocation(mm, mockObject, new Object[] { "1" });
		this.hamspy.invoke(mi);
		assertNull(mi.getReturnValue());
	}

	public void testSetExpectationWithMock() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		IMockObject mockObject1 = EasyMock.createMock(IMockObject.class);
		IMockObject mockObject2 = EasyMock.createMock(IMockObject.class);
		this.hamspy.setExpectation(mm, mockObject1, new Object[] { "1" })
				.setReturnValue("1");
		this.hamspy.setExpectation(mm, mockObject2, new Object[] { "1" })
				.setReturnValue("2");
		MethodInvocation mi = new MethodInvocation(mm, mockObject2,
				new Object[] { "1" });
		this.hamspy.invoke(mi);
		assertEquals("2", mi.getReturnValue());
		mi = new MethodInvocation(mm, mockObject1, new Object[] { "1" });
		this.hamspy.invoke(mi);
		assertEquals("1", mi.getReturnValue());
	}

	public void testSetStubbedExpectation() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		MethodHandler mh = this.hamspy.setStubExpectation(mm).setReturnValue(
				"bar");
		mh.verify();
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);
		assertEquals("bar", mi.getReturnValue());
		mi = new MethodInvocation(mm, mockObject, new Object[] { null });
		this.hamspy.invoke(mi);
		assertEquals("bar", mi.getReturnValue());
		for (int i = 0; i < 1000; i++) {
			mi = new MethodInvocation(mm, mockObject, new Object[] { null });
			this.hamspy.invoke(mi);
			assertEquals("bar", mi.getReturnValue());
			mh.verify();
		}
	}

	public void testSetStubbedExpectationsWithArguments() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		this.hamspy.setStubExpectation(mm, new Object[] { "1" })
				.setReturnValue("1");
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { "1" });
		this.hamspy.invoke(mi);
		assertEquals("1", mi.getReturnValue());
		mi = new MethodInvocation(mm, mockObject, new Object[] { "1" });
		this.hamspy.invoke(mi);
		assertEquals("1", mi.getReturnValue());
		mi = new MethodInvocation(mm, mockObject, new Object[] { "1" });
		this.hamspy.invoke(mi);
		assertEquals("1", mi.getReturnValue());
		mi = new MethodInvocation(mm, mockObject, new Object[] { "2" });
		this.hamspy.invoke(mi);
		assertNull(mi.getReturnValue());
	}

	public void testSetStubbedExpectationsWithMock() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		IMockObject mockObject1 = EasyMock.createMock(IMockObject.class);
		IMockObject mockObject2 = EasyMock.createMock(IMockObject.class);
		this.hamspy.setStubExpectation(mm, mockObject1, new Object[] { "1" })
				.setReturnValue("1");
		this.hamspy.setStubExpectation(mm, mockObject2, new Object[] { "1" })
				.setReturnValue("2");
		MethodInvocation mi = new MethodInvocation(mm, mockObject2,
				new Object[] { "1" });
		this.hamspy.invoke(mi);
		assertEquals("2", mi.getReturnValue());
		mi = new MethodInvocation(mm, mockObject1, new Object[] { "1" });
		this.hamspy.invoke(mi);
		assertEquals("1", mi.getReturnValue());
		mi = new MethodInvocation(mm, mockObject1, new Object[] { "1" });
		this.hamspy.invoke(mi);
		assertEquals("1", mi.getReturnValue());
	}

	public void testGetVerifierForMockAndArguments() {
		MockMethod mm1 = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		MockMethod mm2 = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		IMockObject mockObject1 = EasyMock.createMock(IMockObject.class);
		IMockObject mockObject2 = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi11 = new MethodInvocation(mm1, mockObject1,
				new Object[] { null });
		MethodInvocation mi21 = new MethodInvocation(mm2, mockObject1,
				new Object[] { null });
		MethodInvocation mi12 = new MethodInvocation(mm1, mockObject2,
				new Object[] { null });
		MethodInvocation mi22 = new MethodInvocation(mm2, mockObject2,
				new Object[] { null });
		MethodInvocation mi11b = new MethodInvocation(mm1, mockObject1,
				new Object[] { "foo" });
		this.hamspy.invoke(mi11);
		this.hamspy.invoke(mi11);
		this.hamspy.invoke(mi12);
		this.hamspy.invoke(mi21);
		this.hamspy.invoke(mi22);
		this.hamspy.invoke(mi11b);
		InvocationVerifier verifier = this.hamspy.getExpectation(mm1, mockObject1,
				new Object[] { null });
		assertEquals(2, verifier.getInvocationCount());
		verifier = this.hamspy.getExpectation(mm1, mockObject2,
				new Object[] { null });
		assertEquals(1, verifier.getInvocationCount());
		verifier = this.hamspy.getExpectation(mm2, mockObject1,
				new Object[] { null });
		assertEquals(1, verifier.getInvocationCount());
		verifier = this.hamspy.getExpectation(mm2, mockObject2,
				new Object[] { null });
		assertEquals(1, verifier.getInvocationCount());
		verifier = this.hamspy.getExpectation(mm1, mockObject1,
				new Object[] { "foo" });
		assertEquals(1, verifier.getInvocationCount());
		verifier = this.hamspy.getExpectation(mm1, mockObject2,
				new Object[] { "foo2" });
		assertEquals(0, verifier.getInvocationCount());
		verifier = this.hamspy.getExpectation(mm1, mockObject1,
				new Object[] { "foo2" }).setArgumentMatcher(0,
				new IArgumentMatcher() {
					public boolean areArgumentsEqual(Object argumentExpected,
							Object argumentActual) {
						return true;
					}

				});
		assertEquals(3, verifier.getInvocationCount());
	}

	// Check that we can get a method verifier for verifying the invocations on
	// a spy object.
	public void testGetVerifierForArguments() {
		MockMethod mm1 = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		MockMethod mm2 = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, true);
		IMockObject mockObject1 = EasyMock.createMock(IMockObject.class);
		IMockObject mockObject2 = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi11 = new MethodInvocation(mm1, mockObject1,
				new Object[] { null });
		MethodInvocation mi21 = new MethodInvocation(mm2, mockObject1,
				new Object[] { null });
		MethodInvocation mi12 = new MethodInvocation(mm1, mockObject2,
				new Object[] { null });
		MethodInvocation mi22 = new MethodInvocation(mm2, mockObject2,
				new Object[] { null });
		MethodInvocation mi11b = new MethodInvocation(mm1, mockObject1,
				new Object[] { "foo" });
		this.hamspy.invoke(mi11);
		this.hamspy.invoke(mi11);
		this.hamspy.invoke(mi12);
		this.hamspy.invoke(mi21);
		this.hamspy.invoke(mi22);
		this.hamspy.invoke(mi11b);
		InvocationVerifier verifier = this.hamspy.getExpectation(mm1,
				new Object[] { null });
		assertEquals(3, verifier.getInvocationCount());
		verifier = this.hamspy.getExpectation(mm2, new Object[] { null });
		assertEquals(2, verifier.getInvocationCount());
		verifier = this.hamspy.getExpectation(mm1, new Object[] { "foo" });
		assertEquals(1, verifier.getInvocationCount());
		verifier = this.hamspy.getExpectation(mm1, new Object[] { "foo2" });
		assertEquals(0, verifier.getInvocationCount());
		verifier = this.hamspy.getExpectation(mm1, new Object[] { "foo2" })
				.setArgumentMatcher(0, new IArgumentMatcher() {
					public boolean areArgumentsEqual(Object argumentExpected,
							Object argumentActual) {
						return true;
					}

				});
		assertEquals(4, verifier.getInvocationCount());
	}

	// When mocking a non-abstract class we may want the mock to return default
	// values; not the values that the parent would return.
	// I.e. we want the same behaviour as Mockito.
	public void testMockitoLike() throws Throwable {
		// Indicate that default values must be returned.
		this.hamspy = new Hamspy(true);

		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.BOOLEAN_CLASS, false);
		IMockObject mockObject = EasyMock.createMock(IMockObject.class);
		MethodInvocation mi = new MethodInvocation(mm, mockObject,
				new Object[] { null });
		this.hamspy.invoke(mi);

		// The most important assert; check that Hamspy set that the method has
		// been evaluated.
		assertTrue(mi.isEvaluated());

		// Sanity test.
		assertEquals(new Boolean(false), mi.getReturnValue());
	}

}
