package com.hammingweight.hammock;

import java.io.IOException;
import java.util.Stack;
import java.util.Vector;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;

public class MethodHandlerTest extends TestCase implements IMockObject {

	public static final IMockObject MOCK_OBJECT1 = EasyMock
			.createMock(IMockObject.class);
	public static final IMockObject MOCK_OBJECT2 = EasyMock
			.createMock(IMockObject.class);
	public static final IMockObject MOCK_OBJECT3 = new MethodHandlerTest();

	public void testConstructorNullMethodNotAllowed() {
		MockMethod mm = new MockMethod("foo", 3,
				IClassDefinitions.OBJECT_CLASS, false);
		new MethodHandler(mm);
		try {
			new MethodHandler((MockMethod) null);
			fail();
		} catch (NullPointerException npe) {

		}
	}

	public void testConstructorNullArgsNotAllowed() {
		MockMethod mm = new MockMethod("foo", 3,
				IClassDefinitions.OBJECT_CLASS, false);
		Object[] args = new Object[] { "hello", "world", "1" };

		new MethodHandler(mm, args);
		try {
			new MethodHandler(mm, (Object[]) null);
			fail();
		} catch (NullPointerException npe) {

		}
		try {
			new MethodHandler(null, args);
			fail();
		} catch (NullPointerException npe) {

		}
	}

	public void testConstructorValidateNumberArgs() {
		// Method takes 3 args
		MockMethod mm = new MockMethod("foo", 3,
				IClassDefinitions.OBJECT_CLASS, false);
		// but we supply only 2.
		Object[] args = new Object[] { "hello", "world" };
		try {
			new MethodHandler(mm, args);
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.WRONG_NUMBER_OF_ARGS, he
					.getError());
		}
	}

	public void testConstructorWrongArgumentClass() {
		// Argument is an Integer
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { Integer.class }, new Class[] {}, null, true);
		// But we pass a String.
		Object[] args = new Object[] { "Hello, world" };
		try {
			new MethodHandler(mm, args);
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.WRONG_ARGUMENT_CLASS, he
					.getError());
		}
	}

	public void testConstructorWithMethodInvocation() {
		MockMethod mm = new MockMethod("foo", 0,
				IClassDefinitions.OBJECT_CLASS, false);
		MethodInvocation mi = EasyMock.createMock(MethodInvocation.class);
		EasyMock.expect(mi.getMethod()).andReturn(mm);
		EasyMock.expect(mi.getMethodArguments()).andReturn(new Object[] {});
		EasyMock.expect(mi.getMockObject()).andReturn(MOCK_OBJECT1);
		EasyMock.replay(mi);
		new MethodHandler(mi);

		try {
			// A null method invocation isn't allowed.
			new MethodHandler((MethodInvocation) null);
			fail();
		} catch (NullPointerException npe) {
		}
	}

	public void testSetNumberInvocations() {
		MockMethod mm = new MockMethod("foo", 3,
				IClassDefinitions.OBJECT_CLASS, false);

		// Zero invocations is OK
		MethodHandler mh = new MethodHandler(mm);
		mh.setInvocationCount(0);

		// One invocation is OK.
		mh = new MethodHandler(mm);
		mh.setInvocationCount(1);

		// Check that we return the same method handler after setting the number
		// of invocations.
		mh = new MethodHandler(mm);
		assertEquals(mh, mh.setInvocationCount(2));

		// Can't invoke a method, a negative number of times.
		mh = new MethodHandler(mm);
		try {
			mh.setInvocationCount(-1);
			fail();
		} catch (IllegalArgumentException iae) {

		}
	}

	public void testSetReturnValueForVoidMethod() {
		// The method is of type void.
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, null, true);

		// We can't set a return value.
		try {
			new MethodHandler(mm).setReturnValue(null);
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.CANT_SET_VOID_RETURN_VALUE, he
					.getError());
		}
	}

	public void testSetNumberInvocationsMinMaxValues() {
		MockMethod mm = new MockMethod("foo", 3,
				IClassDefinitions.OBJECT_CLASS, false);

		// Min = max = 0 is OK.
		MethodHandler mh = new MethodHandler(mm);
		assertEquals(mh, mh.setInvocationCount(0, 0));

		// Min = 0 < max = 1 is OK.
		mh = new MethodHandler(mm);
		mh.setInvocationCount(0, 1);

		// Min = 5 < max = 10 is OK.
		mh = new MethodHandler(mm);
		mh.setInvocationCount(5, 10);

		// Min = 2 > max = 1 is NOT OK.
		mh = new MethodHandler(mm);
		try {
			mh.setInvocationCount(2, 1);
			fail();
		} catch (IllegalArgumentException iae) {

		}
	}

	public void testSetNumberInvocationsTwice() {
		MockMethod mm = new MockMethod("foo", 3,
				IClassDefinitions.OBJECT_CLASS, false);
		MethodHandler mh = new MethodHandler(mm);
		mh.setInvocationCount(1, 1);

		// We can't set the number of invocations twice.
		try {
			mh.setInvocationCount(1, 1);
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.NUM_INVOCATIONS_ALREADY_SET,
					he.getError());
		}
	}

	public void testSetNumberInvocationsAfterInvocation() throws Throwable {
		MockMethod mm = new MockMethod("foo", 0,
				IClassDefinitions.OBJECT_CLASS, false);
		MethodHandler mh = new MethodHandler(mm).setReturnValue("");
		MethodInvocation mi = new MethodInvocation(mm, this, new Object[] {});

		// We've invoked the method handler.
		mh.invoke(mi);

		// So we can't set the expected number of invocations now.
		try {
			mh.setInvocationCount(0);
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.NUM_INVOCATIONS_ALREADY_SET,
					he.getError());
		}
	}

	public void testSetReturnValueTwice() {
		MockMethod mm = new MockMethod("foo", 3,
				IClassDefinitions.OBJECT_CLASS, false);
		MethodHandler mh = new MethodHandler(mm);

		// We've specified the return value...
		assertEquals(mh, mh.setReturnValue(null));

		// ... so we can't set it again.
		try {
			mh.setReturnValue("3");
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.RETURN_VALUE_ALREADY_SET, he
					.getError());
		}
	}

	public void testSetReturnValueWrongClass() {
		// A Stack is a Vector so we can return a Stack if the return is Vector.
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, Vector.class, true);
		MethodHandler mh = new MethodHandler(mm);
		mh.setReturnValue(new Stack());

		// A String isn't a Vector, so we can't return a String if the return
		// type is Vector.
		mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo", new Class[] {},
				new Class[] {}, Vector.class, true);
		mh = new MethodHandler(mm);
		try {
			mh.setReturnValue("hello, world.");
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.INCORRECT_RETURN_CLASS, he
					.getError());
		}
	}

	public void testSetThrowableTwice() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, Vector.class, true);
		MethodHandler mh = new MethodHandler(mm);

		// We set the exception thrown.
		assertEquals(mh, mh.setThrowable(new NullPointerException()));

		// We can't set it again.
		try {
			mh.setThrowable(new NullPointerException());
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.RETURN_VALUE_ALREADY_SET, he
					.getError());
		}
	}

	public void testSetThrowableOk() {
		// A method throws an IOException
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] { IOException.class },
				Vector.class, true);
		MethodHandler mh = new MethodHandler(mm);

		// Check that we can set an IOException.
		assertEquals(mh, mh.setThrowable(new IOException()));
	}

	public void testSetThrowableNotOk() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] { RuntimeException.class,
						IOException.class }, Vector.class, true);
		MethodHandler mh = new MethodHandler(mm);
		mh.setThrowable(new IOException());

		mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo", new Class[] {},
				new Class[] { Error.class, IOException.class }, Vector.class,
				true);
		mh = new MethodHandler(mm);

		// ClassNotFoundException is checked and does not extend the specified
		// exceptions,
		// so we can't throw it.
		try {
			mh.setThrowable(new ClassNotFoundException());
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.INCORRECT_THROWABLE_CLASS, he
					.getError());
		}
	}

	public void testThrowableNullNotAllowed() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, Vector.class, true);
		MethodHandler mh = new MethodHandler(mm);
		try {
			mh.setThrowable(null);
			fail();
		} catch (NullPointerException npe) {

		}
	}

	public void testInvokeWithReturnValue() throws Throwable {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, null, true);
		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1,
				new Object[] {});
		MethodHandler mh = new MethodHandler(mm);
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());
		assertNull(mi.getReturnValue());
	}

	public void testInvokeWithException() throws Throwable {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, null, true);
		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1,
				new Object[] {});
		MethodHandler mh = new MethodHandler(mm)
				.setThrowable(new ArithmeticException());
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());
		try {
			mi.getReturnValue();
			fail();
		} catch (ArithmeticException ae) {

		}
	}

	public void testInvokeNoReturnValueSet() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, String.class, true);
		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1,
				new Object[] {});
		MethodHandler mh = new MethodHandler(mm);
		try {
			mh.invoke(mi);
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.NO_RETURN_VALUE_SET, he
					.getError());
		}
	}

	public void testInvokeTwiceUnexpected() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, null, true);
		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1,
				new Object[] {});
		MethodHandler mh = new MethodHandler(mm);
		mh.invoke(mi);

		// The method handler has responded to an invocation, it shouldn't
		// respond again.
		mi = new MethodInvocation(mm, MOCK_OBJECT1, new Object[] {});
		mh.invoke(mi);
		assertFalse(mi.isEvaluated());
	}

	public void testInvokeMinimumExpected() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, null, true);
		MethodHandler mh = new MethodHandler(mm).setInvocationCount(2, 3);

		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1,
				new Object[] {});
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());

		// The method handler has responded to an invocation, it should
		// respond again since the expected number of invocations is in [2,3].
		mi = new MethodInvocation(mm, MOCK_OBJECT1, new Object[] {});
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());
	}

	public void testMaximumExpected() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, null, true);
		MethodHandler mh = new MethodHandler(mm).setInvocationCount(2, 3);

		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT2,
				new Object[] {});
		mh.invoke(mi);

		mi = new MethodInvocation(mm, MOCK_OBJECT2, new Object[] {});
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());

		mi = new MethodInvocation(mm, MOCK_OBJECT2, new Object[] {});
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());

		// Can't invoke a fourth time.
		mi = new MethodInvocation(mm, MOCK_OBJECT2, new Object[] {});
		mh.invoke(mi);
		assertFalse(mi.isEvaluated());
	}

	public void testInvokeWithExpectedArguments() throws Throwable {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { String.class }, new Class[] {}, String.class,
				true);
		Object[] args = { "hello" };
		MethodHandler mh = new MethodHandler(mm, args).setReturnValue("");

		// Right method, wrong args.
		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT2,
				new Object[] { "world" });
		mh.invoke(mi);
		assertFalse(mi.isEvaluated());

		// Right args, wrong method.
		MockMethod mm2 = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { String.class }, new Class[] {}, String.class,
				true);
		mi = new MethodInvocation(mm2, MOCK_OBJECT2, new Object[] { "hello" });
		mh.invoke(mi);
		assertFalse(mi.isEvaluated());

		// Right method, right args.
		mi = new MethodInvocation(mm, MOCK_OBJECT2, new Object[] { "hello" });
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());
		assertEquals("", mi.getReturnValue());

		// Right method, right args but already invoked.
		mi = new MethodInvocation(mm, MOCK_OBJECT2, new Object[] { "hello" });
		mh.invoke(mi);
		assertFalse(mi.isEvaluated());
	}

	public void testInvokeWithMockObject() throws Throwable {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { String.class }, new Class[] {}, String.class,
				true);
		Object[] args = { "hello" };
		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1, args);
		MethodHandler mh = new MethodHandler(mi).setReturnValue("world");

		// Right method, right args, wrong object
		mi = new MethodInvocation(mm, MOCK_OBJECT2, args);
		mh.invoke(mi);
		assertFalse(mi.isEvaluated());

		// Right method, right args, right object (sanity)
		mi = new MethodInvocation(mm, MOCK_OBJECT1, args);
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());
		assertEquals("world", mi.getReturnValue());
	}

	public void testInvokeWithMockObject2() throws Throwable {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { String.class }, new Class[] {}, String.class,
				true);
		Object[] args = { "hello" };
		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1, args);
		MethodHandler mh = new MethodHandler(mi).setReturnValue("");

		// Right object, right args, wrong method.
		MockMethod mm2 = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, String.class, true);
		mi = new MethodInvocation(mm2, MOCK_OBJECT1, new Object[] {});
		mh.invoke(mi);
		assertFalse(mi.isEvaluated());

		// Right method, right args, right object (sanity)
		mi = new MethodInvocation(mm, MOCK_OBJECT1, args);
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());
		assertEquals("", mi.getReturnValue());
	}

	public void testInvokeWithNoReturnValueSpecified() {
		MockMethod mm = new MockMethod("foo", 0,
				IClassDefinitions.OBJECT_CLASS, true);
		MethodHandler mh = new MethodHandler(mm);
		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1,
				new Object[] {});
		try {
			mh.invoke(mi);
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.NO_RETURN_VALUE_SET, he
					.getError());
		}

	}

	// Test that verify fails if a method is invoked less than the expected min
	// number of times.
	public void testVerifyLessThanMinExpected() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, false);
		MethodHandler mh = new MethodHandler(mm).setInvocationCount(2)
				.setReturnValue(new Integer(9));

		MethodInvocation mi = new MethodInvocation(mm, this,
				new Object[] { null });
		mh.invoke(mi);
		assertEquals(new Integer(9), mi.getReturnValue());

		try {
			mh.verify();
			fail();
		} catch (HammockException he) {
			assertEquals(
					IHammockExceptionErrors.METHOD_INVOKED_UNEXPECTED_NUMBER_OF_TIMES,
					he.getError());
		}
	}

	public void testVerifyOk() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, false);
		MethodHandler mh = new MethodHandler(mm).setInvocationCount(2, 3)
				.setReturnValue(new Integer(9));

		MethodInvocation mi = new MethodInvocation(mm, this,
				new Object[] { null });
		mh.invoke(mi);
		assertEquals(new Integer(9), mi.getReturnValue());

		try {
			mh.verify();
			fail();
		} catch (HammockException he) {

		}

		mi = new MethodInvocation(mm, this, new Object[] { null });
		mh.invoke(mi);
		assertEquals(new Integer(9), mi.getReturnValue());
		mh.verify();

		mi = new MethodInvocation(mm, this, new Object[] { null });
		mh.invoke(mi);
		assertEquals(new Integer(9), mi.getReturnValue());
		mh.verify();

		mi = new MethodInvocation(mm, this, new Object[] { null });
		mh.invoke(mi);
		assertFalse(mi.isEvaluated());
	}

	public void testSetDelayForReturnValue() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, false);
		MethodHandler mh = new MethodHandler(mm).setReturnValue(new Integer(1))
				.setDelay(220);

		MethodInvocation mi = new MethodInvocation(mm, this,
				new Object[] { "" });
		long start = System.currentTimeMillis();
		mh.invoke(mi);
		long end = System.currentTimeMillis();
		long delay = end - start;
		assertEquals(new Integer(1), mi.getReturnValue());
		assertTrue(delay >= 200);

	}

	public void testSetDelayForException() throws Throwable {
		MockMethod mm = new MockMethod("foo", 1,
				IClassDefinitions.OBJECT_CLASS, false);
		MethodHandler mh = new MethodHandler(mm).setThrowable(
				new NullPointerException()).setDelay(220);

		MethodInvocation mi = new MethodInvocation(mm, this,
				new Object[] { "" });
		long start = System.currentTimeMillis();
		mh.invoke(mi);
		long end = System.currentTimeMillis();
		long delay = end - start;
		assertTrue(delay >= 200);

	}

	public void testIgnoreArgumentNotOk() {
		MockMethod mm = new MockMethod("foo", 2,
				IClassDefinitions.OBJECT_CLASS, false);
		MethodHandler mh = new MethodHandler(mm).setThrowable(
				new NullPointerException()).setDelay(220);

		try {
			mh.ignoreArgument(-1);
			fail();
		} catch (IllegalArgumentException iae) {
		}

		try {
			mh.ignoreArgument(2);
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.WRONG_NUMBER_OF_ARGS, he
					.getError());
		}
	}

	public void testIgnoreArgumentOk() throws Throwable {
		MockMethod mm = new MockMethod("foo", 2,
				IClassDefinitions.OBJECT_CLASS, false);
		MethodHandler mh = new MethodHandler(mm, this, new Object[] { "hello",
				"world" }).setReturnValue("it works");

		MethodInvocation mi = new MethodInvocation(mm, this, new Object[] {
				"hello", "WORLD" });
		mh.invoke(mi);
		assertFalse(mi.isEvaluated());

		mh.ignoreArgument(1);
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());
		assertEquals("it works", mi.getReturnValue());
	}

	public void testAddArgumentMatcher() {
		MethodHandler mh = new MethodHandler(new MockMethod("foo", 1, null,
				true), new Object[] { "" });
		assertEquals(mh, mh.setArgumentMatcher(0, new IArgumentMatcher() {
			public boolean areArgumentsEqual(Object argumentExpected,
					Object argumentActual) {
				return false;
			}

		}));
	}

	public void testAddArgumentMatcherNullNotAllowed() {
		MethodHandler mh = new MethodHandler(new MockMethod("foo", 1, null,
				true));
		try {
			mh.setArgumentMatcher(0, null);
			fail("Shouldn't be able to add a null matcher.");
		} catch (NullPointerException npe) {

		}
	}

	public void testMatch1Argument() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { Foo.class }, new Class[] {}, Short.class, true);
		MethodHandler mh = new MethodHandler(mm, new Object[] { new Foo() });
		mh.setArgumentMatcher(0, new FooMatcher()).setReturnValue(
				new Short((short) 2));
		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1,
				new Object[] { new Foo() });
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());
	}

	public void testMatch2ArgumentOk() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { Bar.class, Foo.class }, new Class[] {}, null,
				true);
		MethodHandler mh = new MethodHandler(mm, new Object[] { new Bar(3),
				new Foo() });
		mh.setArgumentMatcher(1, new FooMatcher()).setArgumentMatcher(0,
				new BarMatcher());
		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1,
				new Object[] { new Bar(3), new Foo() });
		mh.invoke(mi);
		assertTrue(mi.isEvaluated());
	}

	public void testMatch2ArgumentNotOk() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { Bar.class, Foo.class }, new Class[] {}, null,
				true);
		MethodHandler mh = new MethodHandler(mm, new Object[] { new Bar(3),
				new Foo() });
		mh.setArgumentMatcher(1, new FooMatcher()).setArgumentMatcher(0,
				new BarMatcher());
		MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1,
				new Object[] { new Bar(2), new Foo() });
		mh.invoke(mi);
		assertFalse(mi.isEvaluated());
	}

	public void testArgMatchingIndexNegative() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { Bar.class, Foo.class }, new Class[] {}, null,
				true);
		MethodHandler mh = new MethodHandler(mm);
		try {
			mh.setArgumentMatcher(-1, new BarMatcher());
			fail("An argument index can't be negative.");
		} catch (IllegalArgumentException iae) {

		}
	}

	public void testArgMatchingIndexOutOfBounds() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { Bar.class, Foo.class }, new Class[] {}, null,
				true);
		MethodHandler mh = new MethodHandler(mm);
		try {
			mh.setArgumentMatcher(2, new FooMatcher());
			fail("An argument index >= number of arguments.");
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.WRONG_NUMBER_OF_ARGS, he
					.getError());
		}
	}

	public void testCantSetTwoArgumentMatchersForSameArgument() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { Bar.class, Foo.class }, new Class[] {}, null,
				true);
		MethodHandler mh = new MethodHandler(mm);
		try {
			mh.setArgumentMatcher(1, new FooMatcher()).setArgumentMatcher(1,
					new BarMatcher());
			fail("Shouldn't be able to set argument matcher twice.");
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.ARGUMENT_MATCHER_ALREADY_SET,
					he.getError());
		}
	}

	public void testCantIgnoreSetMatchArgument() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { Bar.class, Foo.class }, new Class[] {}, null,
				true);
		MethodHandler mh = new MethodHandler(mm);
		try {
			mh.ignoreArgument(1).setArgumentMatcher(1, new BarMatcher());
			fail("Shouldn't be able to ignore and set an argument matcher.");
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.ARGUMENT_MATCHER_ALREADY_SET,
					he.getError());
		}
	}

	public void testCantSetMatchIgnoreArgument() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] { Bar.class, Foo.class }, new Class[] {}, null,
				true);
		MethodHandler mh = new MethodHandler(mm);
		try {
			mh.setArgumentMatcher(1, new BarMatcher()).ignoreArgument(1);
			fail("Shouldn't be able to set and ignore an argument matcher.");
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.ARGUMENT_MATCHER_ALREADY_SET,
					he.getError());
		}
	}

	// In Hammock 1.3 if one uses a mock object in a separate thread, the verify
	// method may succeed before the method invocation has completed if a delay
	// has been set. This can be confusing. This verifies that this has been
	// fixed in Hammock 1.4.
	public void testSetDelayVerify() {
		MockMethod mm = new MockMethod(MOCK_OBJECT1.getClass(), "foo",
				new Class[] {}, new Class[] {}, null, true);
		final MethodHandler mh = new MethodHandler(mm);
		mh.setDelay(200);
		final MethodInvocation mi = new MethodInvocation(mm, MOCK_OBJECT1,
				new Object[] {});
		long startTime = System.currentTimeMillis();
		new Thread(new Runnable() {
			public void run() {
				mh.invoke(mi);
			}
		}).start();

		while (System.currentTimeMillis() < startTime + 500) {
			try {
				mh.verify();
				break;
			} catch (HammockException he) {

			}
		}

		assertTrue((System.currentTimeMillis() - startTime) >= 150);
		mh.verify();
	}

	// In Hammock 2.0 we added a constructor that looks at the method and the
	// mock object but not the arguemts.
	public void testMethodObjectConstructor() {
		MockMethod mm = new MockMethod("foo", 1, null, true);
		MethodHandler mh1 = new MethodHandler(mm, MOCK_OBJECT1)
				.setInvocationCount(1, 2);
		MethodInvocation mi1 = new MethodInvocation(mm, MOCK_OBJECT1,
				new Object[] { "foo" });
		mh1.invoke(mi1);
		assertTrue(mi1.isEvaluated());
		MethodInvocation mi2 = new MethodInvocation(mm, MOCK_OBJECT2,
				new Object[] { "foo" });
		mh1.invoke(mi2);
		assertFalse(mi2.isEvaluated());
	}

	public IInvocationHandler getInvocationHandler() {
		return null;
	}

	public void setInvocationHandler(IInvocationHandler handler) {
	}

	private static class Foo {

	}

	private static class Bar {
		private int n;

		public Bar(int n) {
			this.n = n;
		}

		public int getN() {
			return this.n;
		}
	}

	private static class FooMatcher implements IArgumentMatcher {

		@Override
		public boolean areArgumentsEqual(Object argumentExpected,
				Object argumentActual) {
			return ((argumentExpected instanceof Foo) && (argumentActual instanceof Foo));
		}

	}

	private static class BarMatcher implements IArgumentMatcher {

		@Override
		public boolean areArgumentsEqual(Object argumentExpected,
				Object argumentActual) {
			if (!(argumentExpected instanceof Bar)) {
				return false;
			}
			if (!(argumentActual instanceof Bar)) {
				return false;
			}

			Bar bar1 = (Bar) argumentExpected;
			Bar bar2 = (Bar) argumentActual;

			return (bar1.getN() == bar2.getN());
		}

	}
}
