package com.hammingweight.hammock;

import java.io.IOException;
import junit.framework.TestCase;
import org.easymock.EasyMock;

public class MethodInvocationTest extends TestCase implements
		IHammockExceptionErrors, IMockObject {

	private static final MockMethod TEST_METHOD1 = new MockMethod("method1", 0,
			IClassDefinitions.OBJECT_CLASS, false);

	private static final MockMethod TEST_METHOD2 = new MockMethod("method1", 2,
			IClassDefinitions.OBJECT_CLASS, false);

	private static final IMockObject MOCK_OBJECT1 = EasyMock
			.createMock(IMockObject.class);

	private static final IMockObject MOCK_OBJECT2 = EasyMock
			.createMock(IMockObject.class);

	private static final Object[] TEST_ARGS1 = {};

	private static final Object[] TEST_ARGS2 = { "Hello", new Integer(1) };

	private static final Object TEST_RETURN_VALUE = "foo";

	private static final Exception TEST_EXCEPTION = new IOException();

	public void testConstructorOK() {
		new MethodInvocation(TEST_METHOD1, MOCK_OBJECT1, TEST_ARGS1);
		new MethodInvocation(TEST_METHOD2, MOCK_OBJECT2, TEST_ARGS2);
	}

	public void testConstructorNullNotAllowed() {
		try {
			new MethodInvocation(null, MOCK_OBJECT1, TEST_ARGS1);
			fail();
		} catch (NullPointerException npe) {
		}

		try {
			new MethodInvocation(TEST_METHOD1, null, TEST_ARGS1);
			fail();
		} catch (NullPointerException npe) {
		}

		try {
			new MethodInvocation(TEST_METHOD1, MOCK_OBJECT1, null);
			fail();
		} catch (NullPointerException npe) {
		}
}


	public void testConstructorBadNumberOfArgs() {
		try {
			new MethodInvocation(TEST_METHOD1, MOCK_OBJECT1, new Object[]{""});
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.WRONG_NUMBER_OF_ARGS, he.getError());
		}
	}

	// Check that a method is invoked on an object of the expected class.
	public void testConstructorBadObjectClass() {
		MockMethod mm1 = new MockMethod(MethodInvocationTest.class, "MTHD1", new Class[]{}, new Class[]{}, null, true);
		try {
			new MethodInvocation(mm1, MOCK_OBJECT2, new Object[]{});
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.INCORRECT_MOCK_OBJECT, he.getError());
		}
	}
	
	public void testSetReturnValue() throws Throwable {
		MethodInvocation mi1 = new MethodInvocation(TEST_METHOD1, MOCK_OBJECT1,
				TEST_ARGS1);
		mi1.setReturnValue(TEST_RETURN_VALUE);
		Object o = mi1.getReturnValue();
		assertEquals(TEST_RETURN_VALUE, o);
	}

	// We should only allow the return value to be set once.
	public void testSetReturnValue2() throws Throwable {
		MethodInvocation mi1 = new MethodInvocation(TEST_METHOD1, MOCK_OBJECT1,
				TEST_ARGS1);
		mi1.setReturnValue(null);
		mi1.setReturnValue(TEST_RETURN_VALUE);
		Object o = mi1.getReturnValue();
		assertNull(o);
	}

	public void testSetThrowable() throws Throwable {
		MethodInvocation mi1 = new MethodInvocation(TEST_METHOD1, MOCK_OBJECT1,
				TEST_ARGS1);
		mi1.setThrowable(TEST_EXCEPTION);
		try {
			mi1.getReturnValue();
			fail();
		} catch (IOException ioe) {
		}
	}

	public void testSetThrowableNullNotAllowed() {
		MethodInvocation mi1 = new MethodInvocation(TEST_METHOD1, MOCK_OBJECT1,
				TEST_ARGS1);
		try {
			mi1.setThrowable(null);
			fail();
		} catch (NullPointerException npe) {
		}
		assertFalse(mi1.isEvaluated());
	}

	public void testCantSetReturnValueAndThrowable() throws Throwable {
		MethodInvocation mi1 = new MethodInvocation(TEST_METHOD1, MOCK_OBJECT1,
				TEST_ARGS1);
		mi1.setReturnValue(null);
		mi1.setThrowable(TEST_EXCEPTION);
		assertNull(mi1.getReturnValue());
	}

	public void testIsEvaluatedAfterSetValue() {
		MethodInvocation mi1 = new MethodInvocation(TEST_METHOD1, MOCK_OBJECT1,
				TEST_ARGS1);
		assertFalse(mi1.isEvaluated());
		mi1.setReturnValue(null);
		assertTrue(mi1.isEvaluated());
	}

	public void testIsEvaluatedAfterSetThrowable() {
		MethodInvocation mi1 = new MethodInvocation(TEST_METHOD1, MOCK_OBJECT1,
				TEST_ARGS1);
		mi1.setThrowable(TEST_EXCEPTION);
		assertTrue(mi1.isEvaluated());
	}

	public void testGetReturnValue() throws Throwable {
		MethodInvocation mi1 = new MethodInvocation(TEST_METHOD1, MOCK_OBJECT1,
				TEST_ARGS1);
		try {
			mi1.getReturnValue();
			fail();
		} catch (HammockException he) {
			assertEquals(METHOD_CANNOT_BE_EVALUATED, he.getError());
		}
	}

	public IInvocationHandler getInvocationHandler() {
		return null;
	}

	public void setInvocationHandler(IInvocationHandler handler) {
	}
}
